/* Copyright INRIA 2004
**
** This file is part of the Scotch distribution.
**
** The Scotch distribution is libre/free software; you can
** redistribute it and/or modify it under the terms of the
** GNU Lesser General Public License as published by the
** Free Software Foundation; either version 2.1 of the
** License, or (at your option) any later version.
**
** The Scotch distribution is distributed in the hope that
** it will be useful, but WITHOUT ANY WARRANTY; without even
** the implied warranty of MERCHANTABILITY or FITNESS FOR A
** PARTICULAR PURPOSE. See the GNU Lesser General Public
** License for more details.
**
** You should have received a copy of the GNU Lesser General
** Public License along with the Scotch distribution; if not,
** write to the Free Software Foundation, Inc.,
** 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
**
** $Id$
*/
/************************************************************/
/**                                                        **/
/**   NAME       : library_errcom.c                        **/
/**                                                        **/
/**   AUTHOR     : Francois PELLEGRINI                     **/
/**                                                        **/
/**   FUNCTION   : This module redirects the errors        **/
/**                generated by the routines of the        **/
/**                libSCOTCH library to the standard       **/
/**                error processing routines.              **/
/**                                                        **/
/**   DATES      : # Version 3.3  : from : 06 oct 1998     **/
/**                                 to     13 oct 1998     **/
/**                # Version 3.4  : from : 01 nov 2001     **/
/**                                 to     01 nov 2001     **/
/**                                                        **/
/************************************************************/

/*
**  The defines and includes.
*/

#define LIBRARY_ERRCOM
#ifndef SCOTCH_COMMON_EXTERNAL
#define SCOTCH_COMMON_EXTERNAL                    /* Do not redefine errorPrint */
#endif /* SCOTCH_COMMON_EXTERNAL */

#include "module.h"
#include "common.h"
#include "scotch.h"

/********************************/
/*                              */
/* The error handling routines. */
/*                              */
/********************************/

/* This routine prints an error message with
** a variable number of arguments, as printf ()
** does, and exits.
** It returns:
** - EXIT  : in all cases.
*/

void
SCOTCH_errorPrint (
const char * const          errstr,               /*+ printf-like variable argument list */
...)
{
  va_list             errlist;                    /* Argument list of the call */
  char                errbuf[1024];               /* Error buffer              */

  va_start   (errlist, errstr);                   /* Open variable-argument list  */
#if ((defined X_ARCHi586_pc_linux2) || (defined X_ARCHi686_pc_linux2))
  vsnprintf  (errbuf, 1023, errstr, errlist);     /* Write result to buffer       */
#else
  vsprintf   (errbuf, errstr, errlist);           /* Write result to buffer       */
#endif /* X_ARCHi586_pc_linux2 */
  va_end     (errlist);                           /* Close variable-argument list */
  errbuf[1023] = '\0';                            /* Set end of string            */
  errorPrint (errbuf);                            /* Print arguments              */

  exit       (1);
}

/* This routine prints a warning message with
** a variable number of arguments, as printf ()
** does.
** It returns:
** - VOID  : in all cases.
*/

void
SCOTCH_errorPrintW (
const char * const          errstr,               /*+ printf-like variable argument list */
...)
{
  va_list             errlist;                    /* Argument list of the call */
  char                errbuf[1024];               /* Error buffer              */

  va_start    (errlist, errstr);                  /* Open variable-argument list  */
#if ((defined X_ARCHi586_pc_linux2) || (defined X_ARCHi686_pc_linux2))
  vsnprintf   (errbuf, 1023, errstr, errlist);    /* Write result to buffer       */
#else
  vsprintf    (errbuf, errstr, errlist);          /* Write result to buffer       */
#endif /* X_ARCHi586_pc_linux2 */
  va_end      (errlist);                          /* Close variable-argument list */
  errbuf[1023] = '\0';                            /* Set end of string            */
  errorPrintW (errbuf);                           /* Print arguments              */
}
